/* This file is part of the KDE project
   Copyright (C) 2001 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "colorpicker.h"
#include "ksopts.h"

#include <qlayout.h>
#include <qpainter.h>
#include <kvbox.h>
#include <kstyle.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <kpushbutton.h>
//Added by qt3to4:
#include <QFocusEvent>
#include <QKeyEvent>
#include <Q3Frame>
#include <QHBoxLayout>
#include <QMouseEvent>

#include <klocale.h>

using namespace Qt;

ColorPicker::ColorPicker( QWidget *parent, const char *name )
    : KDialog( parent),
      m_foregroundColor( -1 ), m_backgroundColor( -1 )
{
    setCaption(i18n( "Pick Color" ));
    setButtons(KDialog::Ok | KDialog::Cancel);
    setDefaultButton(KDialog::Cancel);
    setModal(true);
    KVBox *mainWidget = new KVBox(this);
    setMainWidget(mainWidget);

    QWidget *sampleBox = new QWidget( mainWidget );
    QHBoxLayout *sampleLayout = new QHBoxLayout( sampleBox );

    QLabel *preview = new QLabel( i18n( "Preview:" ), sampleBox );
    sampleLayout->addWidget( preview );

    m_sample = new QLineEdit( i18n( "Sample Text" ), sampleBox );
    m_sample->setFocusPolicy( NoFocus );
    m_sample->setSizePolicy( QSizePolicy( QSizePolicy::Minimum,
                             m_sample->sizePolicy().verData() ) );
    sampleLayout->addWidget( m_sample );
    sampleLayout->addStretch();

    KHBox *box = new KHBox( mainWidget );
    QLabel *description = new QLabel( i18n( "&Foreground:" ), box );
    ColorBar *foregroundColor = new ColorBar( ksopts->ircColors.toValueVector(), box );
    description->setBuddy( foregroundColor );

    box = new KHBox( mainWidget );
    description = new QLabel( i18n( "&Background:" ), box );
    ColorBar *backgroundColor = new ColorBar( ksopts->ircColors.toValueVector(), box );
    description->setBuddy( backgroundColor );
#warning "kde4: actionButton ? for the moment there is not a method to replace it"
#if 0
    QPushButton *ok = actionButton( KDialog::Ok );
    QPushButton *cancel = actionButton( KDialog::Cancel );

    setTabOrder( foregroundColor, backgroundColor );
    setTabOrder( backgroundColor, ok );
    setTabOrder( ok, cancel );

    ok->setAutoDefault( false );
    cancel->setAutoDefault( false );
#endif
    connect( foregroundColor, SIGNAL( colorPicked( int ) ),
             this, SLOT( setForegroundColor( int ) ) );
    connect( backgroundColor, SIGNAL( colorPicked( int ) ),
             this, SLOT( setBackgroundColor( int ) ) );

    enableButton( ok, false );

    updateSample();
}

QString ColorPicker::colorString() const
{
    assert( m_foregroundColor != -1 );
    QString res( QString::number( m_foregroundColor ) );
    if ( m_backgroundColor != -1 )
    {
        res += ',';
        res += QString::number( m_backgroundColor );
    }
    return res;
}

void ColorPicker::setForegroundColor( int col )
{
    enableButton( ok, true );

    m_foregroundColor = col;
    updateSample();
}

void ColorPicker::setBackgroundColor( int col )
{
    m_backgroundColor = col;
    updateSample();
}

void ColorPicker::updateSample()
{
    QColorGroup cg( colorGroup() );

    QColor col = ksopts->textColor;
    if ( m_foregroundColor != -1 )
        col = ksopts->ircColors[ m_foregroundColor ];

    cg.setColor( QPalette::Foreground, col );
    cg.setColor( QPalette::Text, col );

    if ( m_backgroundColor != -1 )
    {
        col = ksopts->ircColors[ m_backgroundColor ];
        cg.setColor( QPalette::Background, col );
        cg.setColor( QPalette::Base, col );
    }

    m_sample->setPalette( QPalette( cg, cg, cg ) );
}

ColorBar::ColorBar( const QVector<QColor> &colors, QWidget *parent,
                      const char *name )
    : Q3Frame( parent, name, Qt::WStaticContents | Qt::WNoAutoErase ),
      m_currentCell( -1 ), m_focusedCell( - 1 ), m_colors( colors ),
      m_cellSize( 0 )
{
    setFrameStyle( StyledPanel | Sunken );

    updateCellSize();

    setFocusPolicy( StrongFocus );
}

void ColorBar::drawContents( QPainter *p )
{
    int x = contentsRect().x();
    int y = contentsRect().y();
    for ( int i = 0; i < m_colors.size(); ++i, x += m_cellSize )
    {
        bool isCurrentCell = ( m_currentCell != -1 &&
                               i == static_cast<uint>( m_currentCell ) );
        bool isFocusedCell = ( m_focusedCell != -1 &&
                               i == static_cast<uint>( m_focusedCell ) );
        drawCell( p, x, y, m_colors[ i ], QString::number( i ),
                  isFocusedCell, isCurrentCell );
    }
}

void ColorBar::keyPressEvent( QKeyEvent *ev )
{
    if ( m_focusedCell == -1 ) {
        Q3Frame::keyPressEvent( ev );
        return;
    }

    switch ( ev->key() )
    {
        case Qt::Key_Left:
            if ( m_focusedCell > 1 )
                m_focusedCell--;
            update();
            ev->accept();
            return;
        case Qt::Key_Right:
            if (  m_focusedCell  < m_colors.size() - 1 )
                m_focusedCell++;
            update();
            ev->accept();
            return;
        case Qt::Key_Enter:
        case Qt::Key_Return:
        case Qt::Key_Space:
            setCurrentCell( m_focusedCell );
            update();
            ev->accept();
            return;
        default: break;
    }

    Q3Frame::keyPressEvent( ev );
}

void ColorBar::focusInEvent( QFocusEvent *ev )
{
    if ( ev->reason() == QFocusEvent::Tab ||
         ev->reason() == QFocusEvent::Backtab )
        m_focusedCell = 0;
    Q3Frame::focusInEvent( ev );
}

void ColorBar::focusOutEvent( QFocusEvent *ev )
{
    if ( ev->reason() == QFocusEvent::Tab ||
         ev->reason() == QFocusEvent::Backtab ||
         ev->reason() == QFocusEvent::Mouse )
        m_focusedCell = -1;
    Q3Frame::focusOutEvent( ev );
}

void ColorBar::fontChange( const QFont &oldFont )
{
    updateCellSize();
    Q3Frame::fontChange( oldFont );
}

void ColorBar::styleChange( QStyle &oldStyle )
{
    updateCellSize();
    Q3Frame::styleChange( oldStyle );
}

bool ColorBar::focusNextPrevChild( bool next )
{
    if ( next )
    {
        assert( m_focusedCell != -1 );

        if (  m_focusedCell  < m_colors.size() - 1 )
        {
            m_focusedCell++;
            update();
            return true;
        }
        return Q3Frame::focusNextPrevChild( next );
    }

    if ( m_focusedCell > 1 )
    {
        m_focusedCell--;
        update();
        return true;
    }

    return Q3Frame::focusNextPrevChild( next );
}

void ColorBar::mousePressEvent( QMouseEvent *ev )
{
    const QPoint &p = ev->pos();
    if ( contentsRect().contains( p ) )
    {
        m_focusedCell = p.x() / m_cellSize;
        update();
    }

    Q3Frame::mousePressEvent( ev );
}

void ColorBar::mouseReleaseEvent( QMouseEvent *ev )
{
    if ( m_focusedCell != -1 )
    {
        setCurrentCell( m_focusedCell );
        update();
    }
    Q3Frame::mouseReleaseEvent( ev );
}

void ColorBar::updateCellSize()
{
    setLineWidth( style()->pixelMetric( QStyle::PM_DefaultFrameWidth ) );

    QFontMetrics metrics( font() );

    m_cellSize = metrics.width( QString::number( m_colors.size() ) ) +
                 ( s_indicatorSize * 2 ) +
                 ( s_focusSize * 2 ) +
                 ( s_innerMargin * 2 );

    setFixedSize( QSize( ( m_colors.size() * m_cellSize ) + ( frameWidth() * 2 ),
                         m_cellSize + ( frameWidth() * 2 ) ) );
}

void ColorBar::setCurrentCell( int cell )
{
    m_currentCell = cell;
    emit colorPicked( cell );
}

void ColorBar::drawCell( QPainter *p, int x, int y, const QColor &color,
                         const QString &text, bool isFocusedCell,
                         bool isCurrentCell )
{
    p->fillRect( x, y, m_cellSize, m_cellSize, color );

    QColor penColor = black;
    // ### hack
    if ( color.red() < 127 && color.green() < 127 && color.blue() < 127 )
        penColor = white;

    p->setPen( penColor );

    if ( isCurrentCell )
    {
        p->fillRect( x, y, m_cellSize, s_indicatorSize, penColor );
        p->fillRect( x, y + s_indicatorSize,
                     s_indicatorSize, m_cellSize - ( 2 * s_indicatorSize ), penColor );
        p->fillRect( x, y + m_cellSize - s_indicatorSize,
                     m_cellSize, s_indicatorSize, penColor );
        p->fillRect( x + m_cellSize - s_indicatorSize, y + s_indicatorSize,
                     s_indicatorSize, m_cellSize - ( 2 * s_indicatorSize ), penColor );
    }

    if ( isFocusedCell )
    {
        int focusRectSize = m_cellSize - ( 2 * s_indicatorSize );

        p->fillRect( x + s_indicatorSize, y + s_indicatorSize,
                     focusRectSize, s_focusSize, penColor );
        p->fillRect( x + s_indicatorSize, y + s_indicatorSize + s_focusSize,
                     s_focusSize, focusRectSize - ( 2 * s_focusSize ), penColor );
        p->fillRect( x + s_indicatorSize,
                     y + m_cellSize - s_indicatorSize - s_focusSize,
                     focusRectSize, s_focusSize, penColor );
        p->fillRect( x + m_cellSize - s_indicatorSize - s_focusSize,
                     y + s_indicatorSize + s_focusSize,
                     s_focusSize, focusRectSize - ( 2 * s_focusSize ), penColor );
    }

    QFontMetrics metrics( p->font() );

    int offset = ( m_cellSize / 2 ) - ( metrics.width( text ) / 2 );
    p->drawText( x + offset, y + s_focusSize + s_indicatorSize +
                 + metrics.ascent(), text );
}

#include "colorpicker.moc"

/* vim: et sw=4
 */
