/* This file is part of ksirc
   Copyright (c) 2001 Malte Starostik <malte@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

// $Id: ksview.cpp 568256 2006-07-31 16:34:29Z mlaurent $

#include <qclipboard.h>
#include <qdatetime.h>
#include <qregexp.h>
#include <q3dragobject.h>
#include <q3valuestack.h>
#include <q3stylesheet.h>
//Added by qt3to4:
#include <QMouseEvent>
#include <QPixmap>
#include <QDragEnterEvent>
#include <QDragMoveEvent>
#include <QDropEvent>
#include <QUrl>
#include <QTextDocumentFragment>
#include <QScrollBar>

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <krun.h>
#include <kmenu.h>
#include <kstringhandler.h>
#include <knotification.h>
#include <kfiledialog.h>
#include <kio/job.h>
#include <k3urldrag.h>
#include <kiconloader.h>
#include <QTextDocument>

#include "ksopts.h"
#include "ksview.moc"
#include "ksparser.h"
#include "nickColourMaker.h"
#include "ksircprocess.h"

using namespace Qt;

KSircView::KSircView(KSircProcess *proc, QWidget *parent, const char *name)
    : QTextBrowser(parent), m_proc(proc)
{
    setObjectName(name);
    m_acceptFiles = false;
    m_timestamps = false;
    clear();
    setUndoRedoEnabled(false);
    connect(this, SIGNAL(anchorClicked(const QUrl &)),
            this, SLOT(activateAnchor(const QUrl &)));

    QPixmap background = ksopts->backgroundPixmap();
    if ( !background.isNull() )
        viewport()->setBackgroundPixmap( background );

	// ### Qt 4.1
    //setLinkColor( ksopts->linkColor );
}

KSircView::~KSircView()
{
}

void KSircView::clear()
{
    m_lines = 0;
    m_timeStamps.clear();
    QTextBrowser::clear();
}

QString KSircView::makeTimeStamp()
{
    QTime now = QTime::currentTime();
    QString timeStamp = QString::fromLatin1( "[%1:%2:%3] " )
        .arg( QString::number( now.hour() ).rightJustified( 2, '0' ) )
        .arg( QString::number( now.minute() ).rightJustified( 2, '0' ) )
        .arg( QString::number( now.second() ).rightJustified( 2, '0' ) );
    return timeStamp;
}

void KSircView::saveURL( const QString &url )
{
	KUrl kurl( url );

	KFileDialog *dlg = new KFileDialog( QString::null, QString::null /*filter*/, this /*parent*/);

	dlg->setKeepLocation( true );

	dlg->setCaption( i18n( "Save As" ) );

	if ( !kurl.fileName().isEmpty() )
		dlg->setSelection( kurl.fileName() );

	if ( dlg->exec() ) {
		KUrl destURL( dlg->selectedURL() );
		if ( destURL.isValid() ) {
			KIO::Job *job = KIO::copy( kurl, destURL );
			job->setAutoErrorHandlingEnabled( true );
		}
	}

	delete dlg;
}

QString KSircView::addLine(const QString &pixmap, const QColor &color, const QString &_text)
{
    //kDebug(5008) << "Start Text:      " << _text << endl;

    QString richText( "<font color=\"%1\">" );
    richText = richText.arg( color.name() );

    if ( !pixmap.isEmpty() )
        richText.prepend( QString::fromLatin1( "<img src=\"%1\"></img>" ).arg( pixmap ) );

    QString timeStamp = QString::fromLatin1( "<font color=\"%1\">%2</font>" )
        .arg( ksopts->textColor.name() )
        .arg( makeTimeStamp() );
    m_timeStamps.append(timeStamp);
    if ( m_timestamps )
        richText.prepend( timeStamp );

    QString text = Qt::escape(_text);

    // ### a bit of a hack: turn '&lt;nick&gt; message' into
    // <span>&lt;nick&gt;<span> message' . span itself isn't supported but it
    // enforces the creation of separate item objects and hence separate
    // drawing of '<nick>' and 'message' , which is needed for BiDi users,
    // according to UV Kochavi <uv1st@yahoo.com> , to prevent output like
    // '<nick message<' , which is supposedly a bug in Qt's reordering.  The
    // same is done for [nick] and >nick< to catch queries.
    QRegExp bidiRe( "^(&lt;\\S+&gt;)(.+)$" );
    text.replace( bidiRe, QString::fromLatin1( "<span>\\1</span>\\2" ) );
    QRegExp bidiRe2( "^(\\[\\S+\\])(.+)$" );
    text.replace( bidiRe2, QString::fromLatin1( "<span>\\1</span>\\2" ) );
    QRegExp bidiRe3( "^(&gt;\\S+&lt;)(.+)$" );
    text.replace( bidiRe3, QString::fromLatin1( "<span>\\1</span>\\2" ) );

    QRegExp nickCol( "~n(.+)~n" );
    nickCol.setMinimal(true);
    int pos;

    while( (pos = nickCol.search(text)) >= 0 ){
        //kDebug(5008) << "Found nick: " << nickCol.cap(1) << endl;
        QString newText = nickCol.cap(1);
        if( nickCol.cap(1) != m_proc->getNick()){
            QColor col = nickColourMaker::colourMaker()->findFg(nickCol.cap(1));

            newText.prepend(QString("<font color=\"%1\">").arg(col.name()));
            newText.append("</font>");
        }  else {
            QColor col = ksopts->ownNickColor.name();
            if( ! col.isValid())
                nickColourMaker::colourMaker()->findFg(nickCol.cap(1));
            newText.prepend(QString("<font color=\"%1\">").arg(col.name()));
            newText.append("</font>");
            if(ksopts->ownNickBold){
                newText.prepend("<b>");
                newText.append("</b>");
            }
            if(ksopts->ownNickUl){
                newText.prepend("<u>");
                newText.append("</u>");
            }
            if(ksopts->ownNickRev){
                newText.prepend("<r>");
                newText.append("</r>");
            }
        }
        text.replace(pos, nickCol.matchedLength(), newText);
    }

    //kDebug(5008) << "After Text:      " << text << endl;

    KSParser parser;
    richText += parser.parse( text );

    richText += "</font>";


    //kDebug(5008) << "Text:      " << _text << endl;

    setCurrentCharFormat(QTextCharFormat());
    richText = KStringHandler::tagURLs( richText );
    //kDebug(5008) << "Rich text: " << richText << endl;

    append( richText );

    m_lines++;
    if ( ksopts->windowLength && m_lines > ksopts->windowLength )
    {
        while ( m_lines > ksopts->windowLength )
        {
            QTextCursor cursor = textCursor();
            cursor.movePosition(QTextCursor::Start);
            cursor.movePosition(QTextCursor::NextBlock, QTextCursor::KeepAnchor);
            cursor.removeSelectedText();
            m_timeStamps.remove( m_timeStamps.begin() );
            m_lines--;
        }
    }

    if (parser.beeped()) {
        KNotification::event(QString::fromLatin1("BeepReceived"),
                        i18n("Beep Received"),QPixmap(),this);
    }

    QTextCursor cursor = textCursor();
    cursor.movePosition(QTextCursor::End);
    QString logText = cursor.block().text();
    // append timestamp if it's not already there
    if ( ! m_timestamps )
        logText.prepend( makeTimeStamp() );

	scrollToBottom();

    return logText + '\n';
}

void KSircView::addRichText(const QString &_text)
{
    //kDebug(5008) << "Start Text:      " << _text << endl;

    QString text = _text;

    QRegExp re("^(<font color=\"[^\"]+\">\\[[0-9:]+\\] </font>)");
    QString timeStamp;

    if(re.search(text) >= 0){
        timeStamp = re.cap(1);
    }
    else {
	timeStamp = QString::fromLatin1( "<font color=\"%1\">%2</font>" )
	    .arg( ksopts->textColor.name() )
	    .arg( makeTimeStamp() );
	if ( m_timestamps )
	    text.prepend( timeStamp );
    }
    m_timeStamps.append(timeStamp);

    setCurrentCharFormat(QTextCharFormat());
    append( text );

    m_lines++;
    if ( ksopts->windowLength && m_lines > ksopts->windowLength )
    {
        while ( m_lines > ksopts->windowLength )
        {
            QTextCursor cursor = textCursor();
            cursor.movePosition(QTextCursor::Start);
            cursor.movePosition(QTextCursor::NextBlock, QTextCursor::KeepAnchor);
            cursor.removeSelectedText();
            m_timeStamps.remove( m_timeStamps.begin() );
            m_lines--;
        }
    }

	scrollToBottom();
}

void KSircView::enableTimeStamps(bool enable)
{
    if(enable == m_timestamps)
        return;
    setUpdatesEnabled( false );
    m_timestamps = enable;
	const int timeStampLen = makeTimeStamp().length();
	QTextCursor cursor = textCursor();
	cursor.movePosition(QTextCursor::Start);
    QStringList::ConstIterator timeStampIt = m_timeStamps.begin();
    for (; timeStampIt != m_timeStamps.end(); ++timeStampIt )
    {
        if ( enable ) {
            cursor.insertFragment(QTextDocumentFragment::fromHtml(*timeStampIt));
		} else {
			cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, timeStampLen);
			cursor.removeSelectedText();
		}
		cursor.movePosition(QTextCursor::NextBlock);
    }
    setUpdatesEnabled( true );
}

void KSircView::activateAnchor(const QUrl &url)
{
	const QString u = url.toString();
	if (QApplication::keyboardModifiers() & Qt::ShiftModifier)
		saveURL(u);
	else
		openBrowser(u);
}

void KSircView::contextMenuEvent(QContextMenuEvent *e)
{
    const QString url = anchorAt(e->pos());
	if (url.isEmpty())
		return;

	// Adding a nice contextmenu
	KMenu* menu = new KMenu( this );
	menu->addTitle( i18n( "URL" ) );
	QAction * openUrl = menu->addAction( i18n("Open URL") );
	QAction * opendLink = menu->addAction( i18n("Copy Link Address"));
	QAction *ret = menu->exec( e->globalPos() );
	if( ret == openUrl)
		openBrowser( url );
	else if( ret == opendLink )
		copyLinkToClipboard( url );
	delete menu;
}

void KSircView::openBrowser(const QString &url )
{
    (void) new KRun( KUrl( url.startsWith("www") ? QString::fromLatin1("http://") + url : url),this);
}

void KSircView::copyLinkToClipboard( const QString &url )
{
    QApplication::clipboard()->setText( url, QClipboard::Clipboard );
}

QColor KSircView::ircColor(int code)
{
    if (code >= 0 && code < 16)
        return ksopts->ircColors[code];
    return QColor();
}

void KSircView::dragEnterEvent(QDragEnterEvent* event)
{
    event->accept((Q3TextDrag::canDecode(event) ||
                   (m_acceptFiles && K3URLDrag::canDecode(event))) &&
                  (!event->source() || event->source() != viewport()));
}

void KSircView::dragMoveEvent(QDragMoveEvent* event)
{
    event->accept(!event->source() || event->source() != viewport());
}

void KSircView::dropEvent(QDropEvent* event)
{
    QStringList urls;
    QString text;

    if (m_acceptFiles && K3URLDrag::decodeLocalFiles(event, urls))
        emit urlsDropped(urls);
    else if (Q3TextDrag::decode(event, text))
        emit textDropped(text);
}

QVariant KSircView::loadResource(int type, const QUrl &name)
{
	if (type != QTextDocument::ImageResource) {
		return QTextBrowser::loadResource(type, name);
	}

	QString icon = name;
	icon.remove("user|");
	return UserIcon(icon);
}

void KSircView::scrollToBottom()
{
	verticalScrollBar()->setValue(verticalScrollBar()->maximum());
}

void KSircView::resizeEvent(QResizeEvent *e)
{
	QTextBrowser::resizeEvent(e);
	scrollToBottom();
}

// vim: ts=4 sw=4 noet
