/*
 * document.cpp
 *
 * Copyright (c) 2001, 2002, 2003, 2004 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 *
 */
#include "document.h"
#include "article.h"
#include "image.h"
#include "textinput.h"
#include "tools_p.h"

#include <krfcdate.h>
#include <kurl.h>

#include <qdatetime.h>
#include <qdom.h>
#include <q3ptrlist.h>

using namespace RSS;

struct Document::Private : public Shared
{
	Private() : version(v0_90), image(NULL), textInput(NULL), language(en)
	{
	}

	~Private()
	{
		delete textInput;
		delete image;
	}

	Version version;
	QString title;
	QString description;
	KUrl link;
	Image *image;
	TextInput *textInput;
	Article::List articles;
	Language language;
	QString copyright;
	QDateTime pubDate;
	QDateTime lastBuildDate;
	QString rating;
	KUrl docs;
	QString managingEditor;
	QString webMaster;
	HourList skipHours;
	DayList skipDays;
};

Document::Document() : d(new Private)
{
}

Document::Document(const Document &other) : d(0)
{
	*this = other;
}

Document::Document(const QDomDocument &doc) : d(new Private)
{
	QString elemText;
	QDomNode rootNode = doc.documentElement();

	// Determine the version of the present RSS markup.
	QString attr = rootNode.toElement().attribute(QLatin1String("xmlns"), QString::null);
	if (!attr.isNull()) {
		/*
		 * Hardcoding these URLs is actually a bad idea, since the DTD doesn't
		 * dictate a specific namespace. Still, most RSS files seem to use
		 * these two, so I'll go for them now. If it turns out that many
		 * mirrors of this RSS namespace are in use, I'll probably have to
		 * distinguish the RSS versions by analyzing the relationship between
		 * the nodes.
		 */
		if (attr == QLatin1String("http://my.netscape.com/rdf/simple/0.9/"))
			d->version = v0_90;
		else if (attr == QLatin1String("http://purl.org/rss/1.0/")) {
			d->version = v1_0;
		}
	} else {
		attr = rootNode.toElement().attribute(QLatin1String("version"), QString::null);
		if (!attr.isNull()) {
			if (attr == QLatin1String("0.91"))
				d->version = v0_91;
			else if (attr == QLatin1String("0.92"))
				d->version = v0_92;
			else if (attr == QLatin1String("0.93"))
				d->version = v0_93;
			else if (attr == QLatin1String("0.94"))
				d->version = v0_94;
			else if (attr == QLatin1String("2.0") || attr == QString::fromLatin1("2"))
				d->version = v2_0;
		}
	}
	QDomNode channelNode = rootNode.namedItem(QLatin1String("channel"));

	if (!(elemText = extractNode(channelNode, QLatin1String("title"))).isNull())
		d->title = elemText;
	if (!(elemText = extractNode(channelNode, QLatin1String("description"))).isNull())
		d->description = elemText;
	if (!(elemText = extractNode(channelNode, QLatin1String("link"))).isNull())
		d->link = elemText;

	/* This is ugly but necessary since RSS 0.90 and 1.0 have a different parent
	 * node for <image>, <textinput> and <item> than RSS 0.91-0.94 and RSS 2.0.
	 */
	QDomNode parentNode;
	if (d->version == v0_90 || d->version == v1_0)
		parentNode = rootNode;
	else
		parentNode = channelNode;

	QDomNode n = parentNode.namedItem(QLatin1String("image"));
	if (!n.isNull())
		d->image = new Image(n);

	n = parentNode.namedItem(QLatin1String("textinput"));
	if (!n.isNull())
		d->textInput = new TextInput(n);

	// Our (hopefully faster) version of elementsByTagName()
	for (n = parentNode.firstChild(); !n.isNull(); n = n.nextSibling()) {
		const QDomElement e = n.toElement();
		if (e.tagName() == QLatin1String("item"))
			d->articles.append(Article(e));
	}

	if (!(elemText = extractNode(channelNode, QLatin1String("copyright"))).isNull())
		d->copyright = elemText;

	if (!(elemText = extractNode(channelNode, QLatin1String("language"))).isNull()) {
		if (elemText == QLatin1String("af"))
			d->language = af;
		else if (elemText == QLatin1String("sq"))
			d->language = sq;
		else if (elemText == QLatin1String("eu"))
			d->language = eu;
		else if (elemText == QLatin1String("be"))
			d->language = be;
		else if (elemText == QLatin1String("bg"))
			d->language = bg;
		else if (elemText == QLatin1String("ca"))
			d->language = ca;
		else if (elemText == QLatin1String("zh-cn"))
			d->language = zh_cn;
		else if (elemText == QLatin1String("zh-tw"))
			d->language = zh_tw;
		else if (elemText == QLatin1String("hr"))
			d->language = hr;
		else if (elemText == QLatin1String("cs"))
			d->language = cs;
		else if (elemText == QLatin1String("da"))
			d->language = da;
		else if (elemText == QLatin1String("nl"))
			d->language = nl;
		else if (elemText == QLatin1String("nl-be"))
			d->language = nl_be;
		else if (elemText == QLatin1String("nl-nl"))
			d->language = nl_nl;
		else if (elemText == QLatin1String("en"))
			d->language = en;
		else if (elemText == QLatin1String("en-au"))
			d->language = en_au;
		else if (elemText == QLatin1String("en-bz"))
			d->language = en_bz;
		else if (elemText == QLatin1String("en-ca"))
			d->language = en_ca;
		else if (elemText == QLatin1String("en-ie"))
			d->language = en_ie;
		else if (elemText == QLatin1String("en-jm"))
			d->language = en_jm;
		else if (elemText == QLatin1String("en-nz"))
			d->language = en_nz;
		else if (elemText == QLatin1String("en-ph"))
			d->language = en_ph;
		else if (elemText == QLatin1String("en-za"))
			d->language = en_za;
		else if (elemText == QLatin1String("en-tt"))
			d->language = en_tt;
		else if (elemText == QLatin1String("en-gb"))
			d->language = en_gb;
		else if (elemText == QLatin1String("en-us"))
			d->language = en_us;
		else if (elemText == QLatin1String("en-zw"))
			d->language = en_zw;
		else if (elemText == QLatin1String("fo"))
			d->language = fo;
		else if (elemText == QLatin1String("fi"))
			d->language = fi;
		else if (elemText == QLatin1String("fr"))
			d->language = fr;
		else if (elemText == QLatin1String("fr-be"))
			d->language = fr_be;
		else if (elemText == QLatin1String("fr-ca"))
			d->language = fr_ca;
		else if (elemText == QLatin1String("fr-fr"))
			d->language = fr_fr;
		else if (elemText == QLatin1String("fr-lu"))
			d->language = fr_lu;
		else if (elemText == QLatin1String("fr-mc"))
			d->language = fr_mc;
		else if (elemText == QLatin1String("fr-ch"))
			d->language = fr_ch;
		else if (elemText == QLatin1String("gl"))
			d->language = gl;
		else if (elemText == QLatin1String("gd"))
			d->language = gd;
		else if (elemText == QLatin1String("de"))
			d->language = de;
		else if (elemText == QLatin1String("de-at"))
			d->language = de_at;
		else if (elemText == QLatin1String("de-de"))
			d->language = de_de;
		else if (elemText == QLatin1String("de-li"))
			d->language = de_li;
		else if (elemText == QLatin1String("de-lu"))
			d->language = de_lu;
		else if (elemText == QLatin1String("de-ch"))
			d->language = de_ch;
		else if (elemText == QLatin1String("el"))
			d->language = el;
		else if (elemText == QLatin1String("hu"))
			d->language = hu;
		else if (elemText == QLatin1String("is"))
			d->language = is;
		else if (elemText == QLatin1String("id"))
			d->language = id;
		else if (elemText == QLatin1String("ga"))
			d->language = ga;
		else if (elemText == QLatin1String("it"))
			d->language = it;
		else if (elemText == QLatin1String("it-it"))
			d->language = it_it;
		else if (elemText == QLatin1String("it-ch"))
			d->language = it_ch;
		else if (elemText == QLatin1String("ja"))
			d->language = ja;
		else if (elemText == QLatin1String("ko"))
			d->language = ko;
		else if (elemText == QLatin1String("mk"))
			d->language = mk;
		else if (elemText == QLatin1String("no"))
			d->language = no;
		else if (elemText == QLatin1String("pl"))
			d->language = pl;
		else if (elemText == QLatin1String("pt"))
			d->language = pt;
		else if (elemText == QLatin1String("pt-br"))
			d->language = pt_br;
		else if (elemText == QLatin1String("pt-pt"))
			d->language = pt_pt;
		else if (elemText == QLatin1String("ro"))
			d->language = ro;
		else if (elemText == QLatin1String("ro-mo"))
			d->language = ro_mo;
		else if (elemText == QLatin1String("ro-ro"))
			d->language = ro_ro;
		else if (elemText == QLatin1String("ru"))
			d->language = ru;
		else if (elemText == QLatin1String("ru-mo"))
			d->language = ru_mo;
		else if (elemText == QLatin1String("ru-ru"))
			d->language = ru_ru;
		else if (elemText == QLatin1String("sr"))
			d->language = sr;
		else if (elemText == QLatin1String("sk"))
			d->language = sk;
		else if (elemText == QLatin1String("sl"))
			d->language = sl;
		else if (elemText == QLatin1String("es"))
			d->language = es;
		else if (elemText == QLatin1String("es-ar"))
			d->language = es_ar;
		else if (elemText == QLatin1String("es-bo"))
			d->language = es_bo;
		else if (elemText == QLatin1String("es-cl"))
			d->language = es_cl;
		else if (elemText == QLatin1String("es-co"))
			d->language = es_co;
		else if (elemText == QLatin1String("es-cr"))
			d->language = es_cr;
		else if (elemText == QLatin1String("es-do"))
			d->language = es_do;
		else if (elemText == QLatin1String("es-ec"))
			d->language = es_ec;
		else if (elemText == QLatin1String("es-sv"))
			d->language = es_sv;
		else if (elemText == QLatin1String("es-gt"))
			d->language = es_gt;
		else if (elemText == QLatin1String("es-hn"))
			d->language = es_hn;
		else if (elemText == QLatin1String("es-mx"))
			d->language = es_mx;
		else if (elemText == QLatin1String("es-ni"))
			d->language = es_ni;
		else if (elemText == QLatin1String("es-pa"))
			d->language = es_pa;
		else if (elemText == QLatin1String("es-py"))
			d->language = es_py;
		else if (elemText == QLatin1String("es-pe"))
			d->language = es_pe;
		else if (elemText == QLatin1String("es-pr"))
			d->language = es_pr;
		else if (elemText == QLatin1String("es-es"))
			d->language = es_es;
		else if (elemText == QLatin1String("es-uy"))
			d->language = es_uy;
		else if (elemText == QLatin1String("es-ve"))
			d->language = es_ve;
		else if (elemText == QLatin1String("sv"))
			d->language = sv;
		else if (elemText == QLatin1String("sv-fi"))
			d->language = sv_fi;
		else if (elemText == QLatin1String("sv-se"))
			d->language = sv_se;
		else if (elemText == QLatin1String("tr"))
			d->language = tr;
		else if (elemText == QLatin1String("uk"))
			d->language = uk;
		else
			d->language = UndefinedLanguage;
	}

	if (!(elemText = extractNode(channelNode, QLatin1String("pubDate"))).isNull()) {
		time_t _time = KRFCDate::parseDate(elemText);
		/* \bug This isn't really the right way since it will set the date to
		 * Jan 1 1970, 1:00:00 if the passed date was invalid; this means that
		 * we cannot distinguish between that date, and invalid values. :-/
		 */
		d->pubDate.setTime_t(_time);
	}

	if (!(elemText = extractNode(channelNode, QLatin1String("dc:date"))).isNull()) {
		time_t _time = KRFCDate::parseDateISO8601(elemText);
		/* \bug This isn't really the right way since it will set the date to
		 * Jan 1 1970, 1:00:00 if the passed date was invalid; this means that
		 * we cannot distinguish between that date, and invalid values. :-/
		 */
		d->pubDate.setTime_t(_time);
	}

	if (!(elemText = extractNode(channelNode, QLatin1String("lastBuildDate"))).isNull()) {
		time_t _time = KRFCDate::parseDate(elemText);
		d->lastBuildDate.setTime_t(_time);
	}

	if (!(elemText = extractNode(channelNode, QLatin1String("rating"))).isNull())
		d->rating = elemText;
	if (!(elemText = extractNode(channelNode, QLatin1String("docs"))).isNull())
		d->docs = elemText;
	if (!(elemText = extractNode(channelNode, QLatin1String("managingEditor"))).isNull())
		d->managingEditor = elemText;
	if (!(elemText = extractNode(channelNode, QLatin1String("webMaster"))).isNull())
		d->webMaster = elemText;

	n = channelNode.namedItem(QLatin1String("skipHours"));
	if (!n.isNull())
		for (QDomElement e = n.firstChild().toElement(); !e.isNull(); e = e.nextSibling().toElement())
			if (e.tagName() == QLatin1String("hour"))
				d->skipHours.append(e.text().toUInt());

	n = channelNode.namedItem(QLatin1String("skipDays"));
	if (!n.isNull()) {
		Day day;
		QString elemText;
		for (QDomElement e = n.firstChild().toElement(); !e.isNull(); e = e.nextSibling().toElement())
			if (e.tagName() == QLatin1String("day")) {
				elemText = e.text().toLower();
				if (elemText == QLatin1String("monday"))
					day = Monday;
				else if (elemText == QLatin1String("tuesday"))
					day = Tuesday;
				else if (elemText == QLatin1String("wednesday"))
					day = Wednesday;
				else if (elemText == QLatin1String("thursday"))
					day = Thursday;
				else if (elemText == QLatin1String("friday"))
					day = Friday;
				else if (elemText == QLatin1String("saturday"))
					day = Saturday;
				else if (elemText == QLatin1String("sunday"))
					day = Sunday;
				else
					day = UndefinedDay;
				if (day != UndefinedDay)
					d->skipDays.append(day);
			}
	}
}

Document::~Document()
{
	if (d->deref())
		delete d;
}

Version Document::version() const
{
	return d->version;
}

QString Document::verbVersion() const
{
	switch (d->version) {
		case v0_90: return QLatin1String("0.90");
		case v0_91: return QLatin1String("0.91");
		case v0_92: return QLatin1String("0.92");
		case v0_93: return QLatin1String("0.93");
		case v0_94: return QLatin1String("0.94");
		case v1_0: return QLatin1String("1.0");
		case v2_0: return QLatin1String("2.0");
	}
	return QString();
}

QString Document::title() const
{
	return d->title;
}

QString Document::description() const
{
	return d->description;
}

const KUrl &Document::link() const
{
	return d->link;
}

Image *Document::image()
{
	return d->image;
}

const Image *Document::image() const
{
	return d->image;
}

TextInput *Document::textInput()
{
	return d->textInput;
}

const TextInput *Document::textInput() const
{
	return d->textInput;
}

const Article::List &Document::articles() const
{
	return d->articles;
}

Language Document::language() const
{
	return d->language;
}

QString Document::copyright() const
{
	return d->copyright;
}

const QDateTime &Document::pubDate() const
{
	return d->pubDate;
}

const QDateTime &Document::lastBuildDate() const
{
	return d->lastBuildDate;
}

QString Document::rating() const
{
	return d->rating;
}

const KUrl &Document::docs() const
{
	return d->docs;
}

QString Document::managingEditor() const
{
	return d->managingEditor;
}

QString Document::webMaster() const
{
	return d->webMaster;
}

const HourList &Document::skipHours() const
{
	return d->skipHours;
}

const DayList &Document::skipDays() const
{
	return d->skipDays;
}

Document &Document::operator=(const Document &other)
{
	if (this != &other) {
		other.d->ref();
		if (d && d->deref())
			delete d;
		d = other.d;
	}
	return *this;
}

// vim:noet:ts=4
